using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using Microsoft.SharePoint.Taxonomy;

namespace ManagedMetadataField.Features.RegionalOffice
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("f0345dea-b734-491f-8dde-149d444f1303")]
    public class RegionalOfficeEventReceiver : SPFeatureReceiver
    {
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            SPSite site = properties.Feature.Parent as SPSite;
            ConnectTaxonomyField(site, new Guid("{dcb76480-4735-11e0-ab07-0800200c9a66}"), "Global", "Office Locations");
        }

        public static void ConnectTaxonomyField(SPSite site, Guid fieldId, string termGroup, string termSetName)
        {
            if (site.RootWeb.Fields.Contains(fieldId))
            {
                TaxonomySession session = new TaxonomySession(site);

                if (session.DefaultKeywordsTermStore != null)
                {
                    // get the default metadata service application
                    var termStore = session.DefaultKeywordsTermStore;
                    var group = termStore.Groups.GetByName(termGroup);
                    var termSet = group.TermSets.GetByName(termSetName);
                    TaxonomyField field = site.RootWeb.Fields[fieldId] as TaxonomyField;
                    // connect the field to the specified term
                    field.SspId = termSet.TermStore.Id;
                    field.TermSetId = termSet.Id;
                    field.TargetTemplate = string.Empty;
                    field.AnchorId = Guid.Empty;
                    field.Update();
                }
                else
                {
                    throw new TermStoreNotFoundException(string.Format("DefaultKeywordsTermStore not found in site {0}", site.Url));
                }
            }
            else
            {
                throw new ArgumentException(string.Format("Field {0} not found in site {1}", fieldId, site.Url), "fieldId");
            }
        }
    }

    [Serializable]
    public class TermStoreNotFoundException : Exception
    {
        public TermStoreNotFoundException() { }
        public TermStoreNotFoundException(string message) : base(message) { }
        public TermStoreNotFoundException(string message, Exception inner) : base(message, inner) { }
        protected TermStoreNotFoundException(
          System.Runtime.Serialization.SerializationInfo info,
          System.Runtime.Serialization.StreamingContext context)
            : base(info, context) { }
    }

    public static class TaxonomyExtensions
    {
        public static Group GetByName(this GroupCollection groupCollection, string groupName)
        {
            if (String.IsNullOrEmpty(groupName))
            {
                throw new ArgumentException("Taxonomy group name cannot be empty", "name");
            }
            foreach (var group in groupCollection)
            {
                if (group.Name == groupName)
                {
                    return group;
                }
            }
            throw new ArgumentOutOfRangeException("groupName", groupName, "Could not find the taxonomy group");
        }

        public static TermSet GetByName(this TermSetCollection termSets, string termSetName)
        {
            if (String.IsNullOrEmpty(termSetName))
            {
                throw new ArgumentException("Term set name cannot be empty", "name");
            }
            foreach (var termSet in termSets)
            {
                if (termSet.Name == termSetName)
                {
                    return termSet;
                }
            }
            throw new ArgumentOutOfRangeException("termSetName", termSetName, "Could not find the term set");
        }
    }
}
